@@ -65,7 +65,7 @@ struct _FmAppConfig
 {
     FmConfig parent;
     /* config */
-    guint bm_open_method;
+    int bm_open_method;
 
     /* volume */
     gboolean mount_on_startup;
@@ -88,10 +88,16 @@ struct _FmAppConfig
     GtkSortType sort_type;
     int sort_by;
 
+    char* su_cmd;
+
     /* desktop manager */
+    /* FIXME: make these setting per FmDesktop */
     /* emit "changed::wallpaper" */
     FmWallpaperMode wallpaper_mode;
     char* wallpaper;
+    char** wallpapers;
+    int wallpapers_configured;
+    gboolean wallpaper_common;
     GdkColor desktop_bg;
     /* emit "changed::desktop_text" */
     GdkColor desktop_fg;
@@ -100,8 +106,8 @@ struct _FmAppConfig
     char* desktop_font;
 
     gboolean show_wm_menu;
-
-    char* su_cmd;
+    GtkSortType desktop_sort_type;
+    int desktop_sort_by;
 };
 
 struct _FmAppConfigClass
