@@ -2,7 +2,7 @@
  *      desktop.c
  *
  *      Copyright 2010 - 2012 Hong Jen Yee (PCMan) <pcman.tw@gmail.com>
- *      Copyright 2012 Andriy Grytsenko (LStranger) <andrej@rep.kiev.ua>
+ *      Copyright 2012-2013 Andriy Grytsenko (LStranger) <andrej@rep.kiev.ua>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -417,8 +417,8 @@ static void layout_items(FmDesktop* self)
             else
             {
 _next_position:
-                item->x = x;
-                item->y = y;
+                item->x = self->working_area.x + x;
+                item->y = self->working_area.y + y;
                 calc_item_size(self, item, icon);
                 y += self->cell_h;
                 if(y > bottom)
@@ -448,8 +448,8 @@ _next_position:
             else
             {
 _next_position_rtl:
-                item->x = x;
-                item->y = y;
+                item->x = self->working_area.x + x;
+                item->y = self->working_area.y + y;
                 calc_item_size(self, item, icon);
                 y += self->cell_h;
                 if(y > bottom)
@@ -1466,6 +1466,8 @@ static FmDesktopItem* get_nearest_item(FmDesktop* desktop, FmDesktopItem* item,
 
     if(!gtk_tree_model_get_iter_first(model, &it))
         return NULL;
+    if(!item) /* there is no focused item yet, select first one then */
+        return fm_folder_model_get_item_userdata(desktop->model, &it);
 
     min_x_dist = min_y_dist = (guint)-1;
     item2 = NULL;
@@ -1982,9 +1984,11 @@ static gboolean on_single_click_timeout(gpointer user_data)
     GdkWindow* window;
     int x, y;
 
+    if(g_source_is_destroyed(g_main_current_source()))
+        return FALSE;
     window = gtk_widget_get_window(w);
     /* generate a fake button press */
-    /* FIXME: will this cause any problem? */
+    /* FIXME: will this cause any problem? needs to be redesigned later */
     evt.type = GDK_BUTTON_PRESS;
     evt.window = window;
     gdk_window_get_pointer(window, &x, &y, &evt.state);
@@ -2011,7 +2015,7 @@ static gboolean on_motion_notify(GtkWidget* w, GdkEventMotion* evt)
         {
             GtkTreeIter it;
             FmDesktopItem* item = hit_test(self, &it, evt->x, evt->y);
-            GdkWindow* window = gtk_widget_get_window(w);
+            GdkWindow* window;
 
             if(item != self->hover_item)
             {
@@ -2020,21 +2024,22 @@ static gboolean on_motion_notify(GtkWidget* w, GdkEventMotion* evt)
                     g_source_remove(self->single_click_timeout_handler);
                     self->single_click_timeout_handler = 0;
                 }
+                window = gtk_widget_get_window(w);
+                if(item)
+                {
+                    gdk_window_set_cursor(window, hand_cursor);
+                    /* FIXME: timeout should be customizable */
+                    if(self->single_click_timeout_handler == 0)
+                        self->single_click_timeout_handler = g_timeout_add(400, on_single_click_timeout, self); //400 ms
+                        /* Making a loop to aviod the selection of the item */
+                        /* on_single_click_timeout(self); */
+                }
+                else
+                {
+                    gdk_window_set_cursor(window, NULL);
+                }
+                self->hover_item = item;
             }
-            if(item)
-            {
-                gdk_window_set_cursor(window, hand_cursor);
-                /* FIXME: timeout should be customizable */
-                if(self->single_click_timeout_handler == 0)
-                    self->single_click_timeout_handler = g_timeout_add(400, on_single_click_timeout, self); //400 ms
-                    /* Making a loop to aviod the selection of the item */
-                    /* on_single_click_timeout(self); */
-            }
-            else
-            {
-                gdk_window_set_cursor(window, NULL);
-            }
-            self->hover_item = item;
         }
         return TRUE;
     }
@@ -2279,7 +2284,7 @@ static gboolean on_drag_motion (GtkWidget *dest_widget,
     {
         fm_dnd_dest_set_dest_file(desktop->dnd_dest,
                                   item ? item->fi : fm_folder_get_info(desktop_folder));
-        target = gtk_drag_dest_find_target(dest_widget, drag_context, NULL);
+        target = fm_dnd_dest_find_target(desktop->dnd_dest, drag_context);
         if(target != GDK_NONE &&
            fm_dnd_dest_is_target_supported(desktop->dnd_dest, target))
             action = fm_dnd_dest_get_default_action(desktop->dnd_dest, drag_context, target);
@@ -2289,10 +2294,11 @@ static gboolean on_drag_motion (GtkWidget *dest_widget,
     if(desktop->drop_hilight != item)
     {
         FmDesktopItem* old_drop = desktop->drop_hilight;
-        desktop->drop_hilight = item;
+        if(action) /* don't hilight non-dropable item, see #3591767 */
+            desktop->drop_hilight = item;
         if(old_drop)
             redraw_item(desktop, old_drop);
-        if(item)
+        if(item && action)
             redraw_item(desktop, item);
     }
 
