@@ -49,8 +49,6 @@ static void fm_main_win_destroy(GtkObject *object);
 static void fm_main_win_finalize(GObject *object);
 G_DEFINE_TYPE(FmMainWin, fm_main_win, GTK_TYPE_WINDOW);
 
-static gint fm_main_win_new_tab(FmMainWin* win, FmPath* path);
-
 static void update_statusbar(FmMainWin* win);
 
 static gboolean on_focus_in(GtkWidget* w, GdkEventFocus* evt);
@@ -138,8 +136,12 @@ static void fm_main_win_class_init(FmMainWinClass *klass)
 static gboolean idle_focus_view(gpointer user_data)
 {
     FmMainWin* win = (FmMainWin*)user_data;
+    /* window might be destroyed already */
+    if(g_source_is_destroyed(g_main_current_source()))
+        return FALSE;
     if(win->folder_view)
         gtk_widget_grab_focus(GTK_WIDGET(win->folder_view));
+    win->idle_handler = 0;
     return FALSE;
 }
 
@@ -903,7 +905,7 @@ static void on_close_tab(GtkAction* act, FmMainWin* win)
 
 static void on_go(GtkAction* act, FmMainWin* win)
 {
-    fm_main_win_chdir(win, fm_path_entry_get_path(win->location));
+    fm_main_win_chdir_by_name(win, gtk_entry_get_text(GTK_ENTRY(win->location)));
 }
 
 static void _update_hist_buttons(FmMainWin* win)
@@ -1026,7 +1028,7 @@ static void update_statusbar(FmMainWin* win)
         gtk_widget_hide(GTK_WIDGET(win->vol_status));
 }
 
-static gint fm_main_win_new_tab(FmMainWin* win, FmPath* path)
+gint fm_main_win_add_tab(FmMainWin* win, FmPath* path)
 {
     FmTabPage* page = fm_tab_page_new(path);
     GtkWidget* gpage = GTK_WIDGET(page);
@@ -1043,20 +1045,13 @@ static gint fm_main_win_new_tab(FmMainWin* win, FmPath* path)
 
     /* add the tab */
     ret = gtk_notebook_append_page(win->notebook, gpage, GTK_WIDGET(page->tab_label));
+    gtk_widget_show_all(gpage);
     gtk_notebook_set_tab_reorderable(win->notebook, gpage, TRUE);
     gtk_notebook_set_current_page(win->notebook, ret);
 
     return ret;
 }
 
-gint fm_main_win_add_tab(FmMainWin* win, FmPath* path)
-{
-    gint n = fm_main_win_new_tab(win, path);
-
-    gtk_widget_show_all(gtk_notebook_get_nth_page(win->notebook, n));
-    return n;
-}
-
 FmMainWin* fm_main_win_add_win(FmMainWin* win, FmPath* path)
 {
     win = fm_main_win_new();
@@ -1065,7 +1060,7 @@ FmMainWin* fm_main_win_add_win(FmMainWin* win, FmPath* path)
                                 app_config->win_height);
     gtk_widget_show_all(GTK_WIDGET(win));
     /* create new tab */
-    fm_main_win_new_tab(win, path);
+    fm_main_win_add_tab(win, path);
     gtk_window_present(GTK_WINDOW(win));
     return win;
 }
@@ -1148,7 +1143,7 @@ static void on_folder_view_clicked(FmFolderView* fv, FmFolderViewClickType type,
     case FM_FV_CONTEXT_MENU:
         break; /* handled by FmFolderView */
     case FM_FV_MIDDLE_CLICK:
-        if(fm_file_info_is_dir(fi))
+        if(fi && fm_file_info_is_dir(fi))
             fm_main_win_add_tab(win, fm_file_info_get_path(fi));
         break;
     case FM_FV_CLICK_NONE: ;
@@ -1311,6 +1306,8 @@ static void on_notebook_page_removed(GtkNotebook* nb, GtkWidget* page, guint num
     FmTabPage* tab_page = FM_TAB_PAGE(page);
     FmFolderView* folder_view = fm_tab_page_get_folder_view(tab_page);
 
+    g_debug("FmMainWin: removed page %u; %u pages left", num,
+            gtk_notebook_get_n_pages(nb));
     /* disconnect from previous active page */
     g_signal_handlers_disconnect_by_func(tab_page,
                                          on_tab_page_splitter_pos_changed, win);
