@@ -29,6 +29,8 @@
 #include "tab-page.h"
 #include "pcmanfm.h"
 
+#include "gseal-gtk-compat.h"
+
 /* Additional entries for FmFileMenu popup */
 static const char folder_menu_xml[]=
 "<popup>"
@@ -83,19 +85,24 @@ static void fm_tab_page_destroy(GtkWidget *page);
 static void fm_tab_page_destroy(GtkObject *page);
 #endif
 
+static void fm_tab_page_realize(GtkWidget *page);
+static void fm_tab_page_unrealize(GtkWidget *page);
+
 G_DEFINE_TYPE(FmTabPage, fm_tab_page, GTK_TYPE_HPANED)
 
 static void fm_tab_page_class_init(FmTabPageClass *klass)
 {
     GObjectClass *g_object_class = G_OBJECT_CLASS(klass);
-#if GTK_CHECK_VERSION(3, 0, 0)
     GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);
+#if GTK_CHECK_VERSION(3, 0, 0)
     widget_class->destroy = fm_tab_page_destroy;
 #else
     GtkObjectClass *gtk_object_class = GTK_OBJECT_CLASS(klass);
     gtk_object_class->destroy = fm_tab_page_destroy;
 #endif
     g_object_class->finalize = fm_tab_page_finalize;
+    widget_class->realize = fm_tab_page_realize;
+    widget_class->unrealize = fm_tab_page_unrealize;
 
     /* signals that current working directory is changed. */
     signals[CHDIR] =
@@ -171,7 +178,8 @@ void fm_tab_page_destroy(GtkObject *object)
 #endif
 {
     FmTabPage* page = FM_TAB_PAGE(object);
-    /* g_debug("fm_tab_page_destroy"); */
+    g_debug("fm_tab_page_destroy, folder: %s",
+            page->folder ? fm_path_get_basename(fm_folder_get_path(page->folder)) : "(none)");
     free_folder(page);
     if(page->nav_history)
     {
@@ -268,12 +276,40 @@ static FmJobErrorAction on_folder_error(FmFolder* folder, GError* err, FmJobErro
     return FM_JOB_CONTINUE;
 }
 
+static void fm_tab_page_realize(GtkWidget *page)
+{
+    GTK_WIDGET_CLASS(fm_tab_page_parent_class)->realize(page);
+    if (FM_TAB_PAGE(page)->busy)
+        fm_set_busy_cursor(page);
+}
+
+static void fm_tab_page_unrealize(GtkWidget *page)
+{
+    if (FM_TAB_PAGE(page)->busy)
+        fm_unset_busy_cursor(page);
+    GTK_WIDGET_CLASS(fm_tab_page_parent_class)->unrealize(page);
+}
+
+static void _tab_set_busy_cursor(FmTabPage* page)
+{
+    page->busy = TRUE;
+    if (gtk_widget_get_realized(GTK_WIDGET(page)))
+        fm_set_busy_cursor(GTK_WIDGET(page));
+}
+
+static void _tab_unset_busy_cursor(FmTabPage* page)
+{
+    page->busy = FALSE;
+    if (gtk_widget_get_realized(GTK_WIDGET(page)))
+        fm_unset_busy_cursor(GTK_WIDGET(page));
+}
+
 static void on_folder_start_loading(FmFolder* folder, FmTabPage* page)
 {
     FmFolderView* fv = page->folder_view;
     /* g_debug("start-loading"); */
     /* FIXME: this should be set on toplevel parent */
-    fm_set_busy_cursor(GTK_WIDGET(page));
+    _tab_set_busy_cursor(page);
 
 #if FM_CHECK_VERSION(1, 0, 2)
     if(fm_folder_is_incremental(folder))
@@ -334,7 +370,7 @@ static void on_folder_finish_loading(FmFolder* folder, FmTabPage* page)
                   (guint)FM_STATUS_TEXT_NORMAL,
                   page->status_text[FM_STATUS_TEXT_NORMAL]);
 
-    fm_unset_busy_cursor(GTK_WIDGET(page));
+    _tab_unset_busy_cursor(page);
     /* g_debug("finish-loading"); */
 }
 
@@ -433,6 +469,11 @@ static void update_files_popup(FmFolderView* fv, GtkWindow* win,
                                GtkUIManager* ui, GtkActionGroup* act_grp,
                                FmFileInfoList* files)
 {
+    GList* l;
+
+    for(l = fm_file_info_list_peek_head_link(files); l; l = l->next)
+        if(!fm_file_info_is_dir(l->data))
+            return; /* actions are valid only if all selected are directories */
     gtk_action_group_set_translation_domain(act_grp, NULL);
     gtk_action_group_add_actions(act_grp, folder_menu_actions,
                                  G_N_ELEMENTS(folder_menu_actions), win);
@@ -486,14 +527,16 @@ static void fm_tab_page_init(FmTabPage *page)
     gtk_container_set_focus_chain(GTK_CONTAINER(page), focus_chain);
     g_list_free(focus_chain);
 
-    gtk_widget_show_all(GTK_WIDGET(page));
-    if(mode & FM_SP_HIDE)
-        gtk_widget_hide(GTK_WIDGET(page->side_pane));
+//    gtk_widget_show_all(GTK_WIDGET(page));
+//    if(mode & FM_SP_HIDE)
+//        gtk_widget_hide(GTK_WIDGET(page->side_pane));
 
     /* create tab label */
     tab_label = (FmTabLabel*)fm_tab_label_new("");
     gtk_label_set_max_width_chars(tab_label->label, app_config->max_tab_chars);
+#if ! GTK_CHECK_VERSION(3, 0, 0)
     gtk_label_set_ellipsize(tab_label->label, PANGO_ELLIPSIZE_END);
+#endif
     page->tab_label = tab_label;
 
     g_signal_connect(page->folder_view, "sel-changed",
@@ -510,6 +553,7 @@ static void fm_tab_page_init(FmTabPage *page)
     /* the folder view is already loded, call the "loaded" callback ourself. */
     //if(fm_folder_view_is_loaded(folder_view))
     //    on_folder_view_loaded(folder_view, fm_folder_view_get_cwd(folder_view), page);
+    page->busy = FALSE;
 }
 
 FmTabPage *fm_tab_page_new(FmPath* path)
@@ -527,6 +571,10 @@ static void fm_tab_page_chdir_without_history(FmTabPage* page, FmPath* path)
     fm_tab_label_set_text(page->tab_label, disp_name);
     g_free(disp_name);
 
+    char * disp_path = fm_path_display_name(path, FALSE);
+    fm_tab_label_set_tooltip_text(FM_TAB_LABEL(page->tab_label), disp_path);
+    g_free(disp_path);
+
     free_folder(page);
 
     page->folder = fm_folder_from_path(path);
